@@ -550,20 +550,32 @@ static int readheaders(int sock,
 	 * addressed to multiple people on the client machine, there
 	 * will be one copy left in the box for each recipient.  Thus,
 	 * if the mail is addressed to N people, each recipient will
-	 * get N copies.
+	 * get N copies.  This is bad when N > 1.
 	 *
 	 * Foil this by suppressing all but one copy of a message with
-	 * a given Message-ID.  Note: This implementation only catches
-	 * runs of successive identical messages, but that should be
-	 * good enough. 
+	 * a given Message-ID.  The accept_count test ensures that
+	 * multiple pieces of email with the same Message-ID, each
+	 * with a *single* addressee (the N == 1 case), won't be 
+	 * suppressed.
+	 *
+	 * Note: This implementation only catches runs of successive
+	 * messages with the same ID, but that should be good
+	 * enough. A more general implementation would have to store
+	 * ever-growing lists of seen message-IDs; in a long-running
+	 * daemon this would turn into a memory leak even if the 
+	 * implementation were perfect.
 	 * 
-	 * The accept_count test ensures that multiple pieces of identical 
-	 * email, each with a *single* addressee, won't be suppressed.
+	 * Don't mess with this code casually.  It would be way too easy
+	 * to break it in a way that blackholed mail.  Better to pass
+	 * the occasional duplicate than to do that...
 	 */
-	if (MULTIDROP(ctl) && accept_count > 1 && !strncasecmp(line, "Message-ID:", 11))
+	if (MULTIDROP(ctl) && !strncasecmp(line, "Message-ID:", 11))
 	{
 	    if (ctl->lastid && !strcasecmp(ctl->lastid, line))
-		return(PS_REFUSED);
+	    {
+		if (accept_count > 1)
+		    return(PS_REFUSED);
+	    }
 	    else
 	    {
 		if (ctl->lastid)
@@ -625,6 +637,19 @@ static int readheaders(int sock,
 	    continue;
 	}
 
+	/*
+	 * We remove all Delivered-To: headers.
+	 * 
+	 * This is to avoid false mail loops messages when delivering
+	 * local messages to and from a Postfix/qmail mailserver. 
+	 * 
+	 * Should be controlled by an option
+	 */
+	if (ctl->dropdelivered && !strncasecmp(line, "Delivered-To:", 13)) {
+	  free(line);
+	  continue;
+	}
+
 	/*
 	 * If we see a Status line, it may have been inserted by an MUA
 	 * on the mail host, or it may have been inserted by the server
@@ -2176,7 +2201,8 @@ const int maxfetch;		/* maximum number of messages to fetch */
 			    struct idlist	*sdp;
 
 			    for (sdp = ctl->newsaved; sdp; sdp = sdp->next)
-				if (sdp->val.status.num == num)
+				if ((sdp->val.status.num == num)
+						&& (!toolarge || oldmsg))
 				    sdp->val.status.mark = UID_SEEN;
 			}
 
